@@ -14,6 +14,8 @@
 
 #define MSG_413 ("HTTP/1.0 413 Request Entity Too Large\r\nContent-Type: text/html\r\nServer: " SERVER "\r\n\r\n<html><head><title>Request Entity Too Large</head><body><p>Request Entity Too Large.</p></body></html>")
 
+ResponseObject *start_response = NULL;
+
 static inline int 
 blocking_write(client_t *client, char *data, size_t len)
 {
@@ -589,10 +591,6 @@ response_start(client_t *client)
     return ret;
 }
 
-
-
-static ResponseObject *start_response = NULL;
-
 inline void
 setup_start_response(void)
 {
@@ -606,7 +604,7 @@ clear_start_response(void)
 }
 
 
-PyObject *  
+inline PyObject *  
 create_start_response(client_t *cli)
 {
     /*
@@ -614,7 +612,7 @@ create_start_response(client_t *cli)
         start_response->cli = cli;
         return (PyObject *)start_response;
     }
-    yystart_response = PyObject_NEW(ResponseObject, &ResponseObjectType);
+    start_response = PyObject_NEW(ResponseObject, &ResponseObjectType);
     if(start_response == NULL)
         return NULL;
     */
@@ -756,7 +754,7 @@ FileWrapperObject_close(FileWrapperObject *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
-PyObject *
+inline PyObject *
 file_wrapper(PyObject *self, PyObject *args)
 {
     PyObject *filelike = NULL;
@@ -769,7 +767,7 @@ file_wrapper(PyObject *self, PyObject *args)
     return FileWrapperObject_new(self, filelike, blksize);
 }
 
-int 
+inline int 
 CheckFileWrapper(PyObject *obj)
 {
     if (obj->ob_type != &FileWrapperType){
