@@ -35,10 +35,16 @@ typedef struct {
 extern PyTypeObject ResponseObjectType;
 extern PyTypeObject FileWrapperType;
 
-PyObject * create_start_response(client_t *cli);
-PyObject * file_wrapper(PyObject *self, PyObject *args);
+extern ResponseObject *start_response;
 
-int CheckFileWrapper(PyObject *obj);
+inline PyObject * 
+create_start_response(client_t *cli);
+
+inline PyObject * 
+file_wrapper(PyObject *self, PyObject *args);
+
+inline int 
+CheckFileWrapper(PyObject *obj);
 
 inline int
 response_start(client_t *client);
@@ -52,7 +58,7 @@ setup_start_response(void);
 inline void
 clear_start_response(void);
 
-void
+inline void
 send_error_page(client_t *client);
 
 
