@@ -11,12 +11,12 @@
 #include "http_request_parser.h"
 #include "response.h"
 #include "log.h"
-
+#include "client.h"
 
 #define MAX_FDS 1024 * 8
 #define ACCEPT_TIMEOUT_SECS 1
 #define SHORT_TIMEOUT_SECS 2
-#define WRITE_TIMEOUT_SECS 15
+#define WRITE_TIMEOUT_SECS 60 * 5 
 #define READ_LONG_TIMEOUT_SECS 5
 
 #define BACKLOG 1024 * 8
@@ -43,12 +43,18 @@ int max_content_length = 1024 * 1024 * 16; //max_content_length
 
 static char *unix_sock_name = NULL;
 
+static PyObject *switch_value;
+
+
 static void
 r_callback(picoev_loop* loop, int fd, int events, void* cb_arg);
 
 static void
 w_callback(picoev_loop* loop, int fd, int events, void* cb_arg);
 
+static inline void
+resume_wsgi_app(ClientObject *pyclient, picoev_loop* loop);
+
 
 static inline int
 setsig(int sig, void* handler)
@@ -163,35 +169,95 @@ close_conn(client_t *cli, picoev_loop* loop)
 
 }
 
-
 static inline int
-process_wsgi_app(client_t *cli)
+process_resume_wsgi_app(ClientObject *pyclient)
 {
-    PyObject *args = NULL, *start = NULL;
-    
-    start = create_start_response(cli);
+    PyObject *res = NULL;
 
-    if(!start)
-        return 0;
-    args = Py_BuildValue("(OO)", cli->environ, start);
-    cli->response = PyObject_CallObject(wsgi_app, args);
+    client_t *old_client;
+    client_t *client = pyclient->client;
     
-    Py_DECREF(args);
+    //swap bind client_t
+
+    old_client = start_response->cli;
+    start_response->cli = client;
+    
+    res = PyGreenlet_Switch(pyclient->greenlet, pyclient->args, pyclient->kwargs);
+    start_response->cli = old_client;
+    
+    Py_XDECREF(pyclient->args);
+    Py_XDECREF(pyclient->kwargs);
 
     //check response & Py_ErrorOccued
-    if(cli->response && cli->response == Py_None){
+    if(res && res == Py_None){
         PyErr_SetString(PyExc_Exception, "response must be a iter or sequence object");
     }
 
     if (PyErr_Occurred()){ 
         write_error_log(__FILE__, __LINE__);
+        return -1;
+    }
+    
+    if(PyInt_Check(res)){
+        // suspend process
         return 0;
     }
+
+    client->response = res;
+    //next send response 
+    return 1;
+    
+}
+
+static inline int
+process_wsgi_app(client_t *cli)
+{
+    PyObject *args = NULL, *start = NULL, *res = NULL;
+    
+    start = create_start_response(cli);
+
+    if(!start){
+        return -1;
+    }
+    args = Py_BuildValue("(OO)", cli->environ, start);
+
+    res = PyObject_CallObject(wsgi_app, args);
+    Py_DECREF(args);
+    
+
+    //check response & Py_ErrorOccued
+    if(res && res == Py_None){
+        PyErr_SetString(PyExc_Exception, "response must be a iter or sequence object");
+    }
+
+    if (PyErr_Occurred()){ 
+        write_error_log(__FILE__, __LINE__);
+        return -1;
+    }
+    
+    if(PyInt_Check(res)){
+        // suspend process
+        return 0;
+    }
+
+    //next send response 
+    cli->response = res;
     
     return 1;
     
 }
 
+static void
+resume_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
+{
+    ClientObject *pyclient = (ClientObject *)(cb_arg);
+    client_t *client = pyclient->client;
+    picoev_del(loop, client->fd);
+    // resume
+    resume_wsgi_app(pyclient, loop);
+    pyclient->resumed = 0;
+}
+
 static void
 w_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 {
@@ -223,16 +289,66 @@ w_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
     }
 }
 
+static inline void
+resume_wsgi_app(ClientObject *pyclient, picoev_loop* loop)
+{
+    int ret;
+    client_t *client = pyclient->client;
+    ret = process_resume_wsgi_app(pyclient);
+    switch(ret){
+        case -1:
+            //Internal Server Error
+            client->bad_request_code = 500;
+            send_error_page(client);
+            close_conn(client, loop);
+            return;
+        case 0:
+            // suspend
+            return;
+        default:
+            pyclient->client = NULL;
+            break;
+    }
+    
+    ret = response_start(client);
+    switch(ret){
+        case -1:
+            // Internal Server Error
+            client->bad_request_code = 500;
+            send_error_page(client);
+            close_conn(client, loop);
+            return;
+        case 0:
+            // continue
+            // set callback
+#ifdef DEBUG
+            printf("set write callback %d \n", ret);
+#endif
+            picoev_add(loop, client->fd, PICOEV_WRITE, WRITE_TIMEOUT_SECS, w_callback, (void *)client);
+            return;
+        default:
+            // send OK
+            close_conn(client, loop);
+    }
+}
+
 static inline void
 call_wsgi_app(client_t *client, picoev_loop* loop)
 {
     int ret;
-    if(!process_wsgi_app(client)){
-        //Internal Server Error
-        client->bad_request_code = 500;
-        send_error_page(client);
-        close_conn(client, loop);
-        return;
+    ret = process_wsgi_app(client);
+    switch(ret){
+        case -1:
+            //Internal Server Error
+            client->bad_request_code = 500;
+            send_error_page(client);
+            close_conn(client, loop);
+            return;
+        case 0:
+            // suspend
+            return;
+        default:
+            break;
     }
     
     ret = response_start(client);
@@ -442,7 +558,10 @@ setup_server_env(void)
     cache_time_init();
     setup_static_env(server_name, server_port);
     setup_start_response();
+    setup_client();
     PycString_IMPORT;
+    PyGreenlet_Import();
+    switch_value = Py_BuildValue("(i)", -1);
 }
 
 static inline int 
@@ -691,7 +810,8 @@ meinheld_run_loop(PyObject *self, PyObject *args)
     while (loop_done) {
         picoev_loop_once(main_loop, 10);
         i++;
-        if(watchdog && i > 10){
+        // watchdog slow.... skip check
+        if(watchdog && i > 15){
             watchdog_result = PyObject_CallFunction(watchdog, NULL);
             if(PyErr_Occurred()){
                 PyErr_Print();
@@ -800,6 +920,97 @@ meinheld_set_process_name(PyObject *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyObject *
+meinheld_suspend_client(PyObject *self, PyObject *args)
+{
+    PyObject *temp;
+    ClientObject *pyclient;
+    PyGreenlet *parent;
+
+    if (!PyArg_ParseTuple(args, "O:_suspend_client", &temp)){
+        return NULL;
+    }
+    
+    // check client object
+    if(!CheckClientObject(temp)){
+        PyErr_SetString(PyExc_TypeError, "must be a client object");
+        return NULL;
+    }
+
+    pyclient = (ClientObject *)temp;
+    
+    if(!pyclient->greenlet){
+        PyErr_SetString(PyExc_ValueError, "greenlet is not set");
+        return NULL;
+    }
+
+    if(pyclient->resumed == 1){
+        //call later
+        PyErr_SetString(PyExc_Exception, "not called resume ");
+        return NULL;
+    }
+    
+    if(pyclient->client && !(pyclient->suspended)){
+        pyclient->suspended = 1;
+        parent = PyGreenlet_GET_PARENT(pyclient->greenlet);
+        return PyGreenlet_Switch(parent, switch_value, NULL);
+    }else{
+        PyErr_SetString(PyExc_Exception, "already suspended");
+        return NULL;
+    }
+    Py_RETURN_NONE;
+}
+
+PyObject *
+meinheld_resume_client(PyObject *self, PyObject *args)
+{
+    PyObject *temp, *switch_args, *switch_kwargs;
+    ClientObject *pyclient;
+    client_t *client;
+
+    if (!PyArg_ParseTuple(args, "O|OO:_resume_client", &temp, &switch_args, &switch_kwargs)){
+        return NULL;
+    }
+
+    // check client object
+    if(!CheckClientObject(temp)){
+        PyErr_SetString(PyExc_TypeError, "must be a client object");
+        return NULL;
+    }
+
+    pyclient = (ClientObject *)temp;
+    
+    if(!pyclient->greenlet){
+        PyErr_SetString(PyExc_ValueError, "greenlet is not set");
+        return NULL;
+    }
+
+    if(!pyclient->suspended){
+        // not suspend
+        PyErr_SetString(PyExc_Exception, "not suspended");
+        return NULL;
+    }
+
+    if(pyclient->client && !pyclient->resumed){
+        if(switch_args){
+            pyclient->args = switch_args;
+            Py_INCREF(pyclient->args);
+        }
+        if(switch_kwargs){
+            pyclient->kwargs = switch_kwargs;
+            Py_INCREF(pyclient->kwargs);
+        }
+        client = pyclient->client;
+
+        pyclient->suspended = 0;
+        pyclient->resumed = 1;
+        picoev_add(main_loop, client->fd, PICOEV_WRITE, 0, resume_callback, (void *)pyclient);
+    }else{
+        PyErr_SetString(PyExc_Exception, "already resumed");
+        return NULL;
+    }
+    Py_RETURN_NONE;
+}
 
 static PyMethodDef WsMethods[] = {
     {"listen", meinheld_listen, METH_VARARGS, "set host and port num"},
@@ -815,14 +1026,19 @@ static PyMethodDef WsMethods[] = {
     {"set_listen_socket", meinheld_set_listen_socket, METH_VARARGS, "set listen_sock"},
     {"set_watchdog", meinheld_set_watchdog, METH_VARARGS, "set watchdog"},
     {"run", meinheld_run_loop, METH_VARARGS, "set wsgi app, run the main loop"},
+    // green
+    {"_suspend_client", meinheld_suspend_client, METH_VARARGS, "resume client"},
+    {"_resume_client", meinheld_resume_client, METH_VARARGS, "resume client"},
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
 PyMODINIT_FUNC
 initserver(void)
 {
-    (void) Py_InitModule("meinheld.server", WsMethods);
+    (void)Py_InitModule("meinheld.server", WsMethods);
     PyType_Ready(&FileWrapperType);
+    PyType_Ready(&ClientObjectType);
+
 }
 
 
